function [A, S, T, At, St, Tt, varexpl]=SICA(X,noc,varargin)

% Usage:
%   [A, S, T, At, St, Tt, varexpl]=SICA(X,noc,opts)
%
% Input:
% X             I x J matrix
% noc           number of components
% opts. 
%    A          Initial A
%    S          Initial S
%    T          Initial T
%    disregard  index of region to disregard, i.e. active part:  
%                (Still needs to be implemented for T update)
%               X(:,disregard+1:end-disregard)
%    maxiter    maximum number of iteration (default: 1000)
%    conv_crit  Convergence Criterion (default relative change of costfunction of 1e-6)
%    ConstA     1: Constant A else estiamte
%    ConstS     1: Constant S
%    ConstT     1: Constant T
%    dispiter   1: Display result for iteration 
%
% Output:
%           ICA rotated Solution (SICA) (unique solution) 
% A             I x noc mixing matrix 
% S             noc x J matrix of sources
% T             I x noc matrix of time delays
%           Shift invariant subspace analysis (SISA) (ambiguous) 
% At            I x noc mixing matrix 
% St            noc x J matrix of sources
% Tt            I x noc matrix of time delays
%     
% varexpl   Variation explained for SICA solution
%
% Copyright (C) Morten Mrup, Kristoffer Hougaard Madsen and Technical University of Denmark, 
% March 2007
  
if nargin>=3, opts = varargin{1}; else opts = struct; end
varexplold=0;
runit=mgetopt(opts,'runit',0);
if ~isfield(opts,'S') & runit~=1 
    disp(['Finding the best out of ' num2str(10) ' initial solution'])
    for k=1:10
        if k==1
            disp(['Now Estimating ' num2str(k) 'st solution ']);
        elseif k==2
            disp(['Now Estimating ' num2str(k) 'nd solution ']);
        elseif k==3
            disp(['Now Estimating ' num2str(k) 'rd solution ']);
        else
            disp(['Now Estimating ' num2str(k) 'th solution ']);
        end
        optsn=opts;
        optsn.dispiter=0;
        optsn.runit=1;    
        optsn.maxiter=25;
        [Aq, Sq, Tq, n1, n2, n3, varexpl]=SICA(X,noc,optsn);
        if varexpl>varexplold
            disp(['Variation explained by best solution ' num2str(varexpl) ]);
            varexplold=varexpl;
            A=Aq;
            S=Sq;
            T=Tq;
        end
    end
else
    mx=max(abs(X(:)));
    A=mgetopt(opts,'A',mx*randn(size(X,1),noc));
    S=mgetopt(opts,'S',mx*randn(noc,size(X,2)));
    T=mgetopt(opts,'T',zeros(size(A)));
end

nyT=mgetopt(opts,'nyT',1);
maxiter=mgetopt(opts,'maxiter',1000);
conv_crit=mgetopt(opts,'convcrit',1e-6);
disregard=mgetopt(opts,'disregard',0);
betamax=mgetopt(opts,'beta',0);
beta=0;
gamma=mgetopt(opts,'gamma',0);
SST=norm(X,'fro')^2;
constS=mgetopt(opts,'constS',0);
constA=mgetopt(opts,'constA',0);
constT=mgetopt(opts,'constT',0);
dispiter=mgetopt(opts,'dispiter',1);
N=size(X,2);
f=j*2*pi*[0:N-1]/N;
W=ones(1,size(X,2));
W(1:disregard)=0;
W(end:-1:end-disregard+1)=0;
Wf=fft(W,[],2);
Wf=Wf(:,1:floor(size(Wf,2)/2)+1);
Xf=fft(X,[],2);
Xf=Xf(:,1:floor(size(Xf,2)/2)+1);
Sf=fft(S,[],2);
Sf=Sf(:,1:floor(size(Sf,2)/2)+1);
f=f(1:size(Xf,2));


% Initial Cost
for i=1:size(A,1)
   Sft=Sf.*exp(T(i,:)'*f); 
   Sft=[Sft conj(Sft(:,end-1:-1:2))];
   S=real(ifft(Sft,[],2));
   Rec(i,:)=A(i,:)*S;
end    
cost=0.5*norm(X(:,disregard+1:end-disregard)-Rec(:,disregard+1:end-disregard),'fro')^2;
cost_oldt=cost;
dcost=inf;
varexpl=(SST-2*cost)/SST;
told=cputime;
iter=0;


% Display algorithm progress
if dispiter
    disp([' '])
    disp(['Shifted Factor Analysis'])
    disp(['A ' num2str(noc) ' component model will be fitted']);
    disp(['Sparseness imposed on frequency transformed S : ' num2str(beta)])
    disp(['L2 regularization imposed on A : ' num2str(gamma)])
    disp(['To stop algorithm press control C'])
    disp([' ']);
    dheader = sprintf('%12s | %12s | %12s | %12s | %12s | %12s | %12s','Iteration','Expl. var.','Cost func.','Delta costf.',' Time(s)   ');
    dline = sprintf('-------------+--------------+--------------+--------------+--------------+');
    disp(dline);
    disp(dheader);
    disp(dline);
end

while iter<maxiter & (dcost>=cost*conv_crit | mod(iter,10)==0 ) 

    iter=iter+1;
    
    if mod(iter,10)==0 & dispiter
        disp(dline);
        disp(dheader);
        disp(dline);
    end
        
    % Update S
    if ~constS
        for i=1:size(Sf,2)           
            Sf(:,i)=pinv(A.*exp(T*f(i)))*Xf(:,i);
        end
    end
     
    % Update A and calculate cost function
     if ~constA | (~constS & constT) 
        for i=1:size(A,1)
           Sft=Sf.*exp(T(i,:)'*f); 
           Sft=[Sft conj(Sft(:,end-1:-1:2))];
           S=real(ifft(Sft,[],2));
           if ~constA
               A(i,:)=X(i,disregard+1:end-disregard)*S(:,disregard+1:end-disregard)'*...
                   inv(S(:,disregard+1:end-disregard)*S(:,disregard+1:end-disregard)');
           end
           if  (~constS & constT)
                Rec(i,:)=A(i,:)*S;
           end
        end
     end

    % Update T
    if ~constT
        if mod(iter,10)==0 & iter~=maxiter 
            if runit~=1 
                disp('Re-estimating shifts by cross-correlation');
            end
            [T,A]=estTimeAutCor(Xf,A,Sf,T,f,N);
        end
        P.A=A;
        P.Xf=Xf;
        P.Sf=Sf;
        P.N2=size(X,2);
        P.w=ones(1,length(f));
        P.w(2:end-1)=2;
        P.f=f;
        P.nyT=nyT;
        [T,nyT,cost]=update_TmH(T,P);
        cost=cost/size(S,2); % Use Parseval identity for cost function
        sse=2*cost;
    else
        sse=norm(X(:,disregard+1:end-disregard)-Rec(:,disregard+1:end-disregard),'fro')^2;
        cost=0.5*sse;
    end
    dcost=cost_oldt-cost;
    cost_oldt=cost;
    varexpl=(SST-sse)/SST;
    if rem(iter,1)==0 & dispiter
        t=cputime;
        tim=t-told;
        told=t;
        disp(sprintf('%12.0f | %12.4f | %12.4f | %12.4f | %12.4f ',iter, varexpl,cost,dcost,tim));
    end
end



% Calculate S
if mod(N,2)==0
    Sft=[Sf conj(Sf(:,end-1:-1:2))];
else
    Sft=[Sf conj(Sf(:,end:-1:2))];
end
S=real(ifft(Sft,[],2));

% Normalize solution and align S and T
[At,St]=normalizeSolution(A,S);
tmean=mean(T);
T=T-repmat(tmean,[size(T,1),1]);
Tt=T;
Sf=fft(St,[],2);
Sf=Sf(:,1:floor(size(Sf,2)/2)+1);
Sf=Sf.*exp(tmean'*f);
if mod(N,2)==0
    Sft=[Sf conj(Sf(:,end-1:-1:2))];
else
    Sft=[Sf conj(Sf(:,end:-1:2))];
end
St=real(ifft(Sft,[],2));


% Perform ICA
if ~constS  & ~runit         
     disp('Performing ICA on shift invariant subspace')
     % Save variables to make them accessible for fminunc 
     save('tempres','Sf','f');  

     % Initialize ICA solution
     W0=eye(size(S,1)); 
     W0=[W0; zeros(1,size(W0,2))];
     
     % Find rotation of ICA
     [W,fval,exitflag,output] = fminunc(@icaMLcomp,W0);
     
     % Display rotation result
     disp('Rotation matrix: ');
     disp(W(1:size(Sf,1),:));
     disp('Shift delays: ');
     disp(W(size(Sf,1)+1:end,:));
     
     % Rotate solution
     Sf=W(1:size(Sf,1),:)*(exp(W(size(Sf,1)+1:end,:)'*f).*Sf);
     
%     for k=1:size(Sf,2)
%         Sf(:,k)=(W(1:size(Sf,1),:).*(exp(W(size(Sf,1)+1:end,:)*f(k))))*Sf(:,k);
%     end
     A=At*inv(W(1:size(Sf,1),:));
     
     % Calculate S for rotated solutioin
     if mod(N,2)==0
         Sft=[Sf conj(Sf(:,end-1:-1:2))];
     else
         Sft=[Sf conj(Sf(:,end:-1:2))];
     end     
     S=real(ifft(Sft,[],2));
 end

% Reestimate delays after ICA using Autocorrelations
if ~constS & ~runit  
  T=zeros(size(A));
  for repeat=1:5
      [T,A]=estTimeAutCor(Xf,A,Sf,T,f,N);
  end

  % Use estimated rotated solution to find best A and T
  opts.T=T;
  opts.A=A;
  opts.S=S;
  opts.constS=1;
  [A, S, T,n1,n2,n3,varexpl]=SICA(X,noc,opts);
  
  % Align S and T
  tmean=mean(T);
  T=T-repmat(tmean,[size(T,1),1]);
  Sf=fft(S,[],2);
  Sf=Sf(:,1:floor(size(Sf,2)/2)+1);
  Sf=Sf.*exp(tmean'*f);
  if mod(N,2)==0
      Sft=[Sf conj(Sf(:,end-1:-1:2))];
  else
      Sft=[Sf conj(Sf(:,end:-1:2))];
  end
  S=real(ifft(Sft,[],2));
end



% -------------------------------------------------------------------------
% Parser for optional arguments
function var = mgetopt(opts, varname, default, varargin)
if isfield(opts, varname)
    var = getfield(opts, varname); 
else
    var = default;
end
for narg = 1:2:nargin-4
    cmd = varargin{narg};
    arg = varargin{narg+1};
    switch cmd
        case 'instrset',
            if ~any(strcmp(arg, var))
                fprintf(['Wrong argument %s = ''%s'' - ', ...
                    'Using default : %s = ''%s''\n'], ...
                    varname, var, varname, default);
                var = default;
            end
        otherwise,
            error('Wrong option: %s.', cmd);
    end
end

% -------------------------------------------------------------------------
% Normalizes S with respect to A
function [A,S]=normalizeSolution(A,S)
d=sqrt(sum(S.^2,2));
S=S./repmat(d,[1,size(S,2)]);
A=A.*repmat(d',[size(A,1),1]);





% -------------------------------------------------------------------------
% returns the negative log likelihood of the ICA ML cost function
function logP=icaMLcomp(WT)
load tempres

W=WT(1:size(Sf,1),:);
T=WT(size(Sf,1)+1,:);
logP=0;
for k=1:size(Sf,2)
    Tf=exp(f(k)*T);
    X(:,k)=W*(Sf(:,k).*Tf.');
    % negative log likelihood function
    if k>2 & k<size(Sf,2)
        logP=logP-2*(log(abs(det(W))) - sum(abs(X(:,k))));
    else
        logP=logP-(log(abs(det(W))) - sum(abs(X(:,k)))); 
     end
end

% -------------------------------------------------------------------------
% Function to update T using Newton-Raphson
function [T,nyT,cost]=update_TmH(T,P)
    nyT=P.nyT;
    Sf=P.Sf;
    A=P.A;
    N2=P.N2;
    Xf=P.Xf;
    f=P.f;
    w=P.w;
    Recfd=zeros(size(A,1),size(Sf,2),size(A,2));
    for d=1:size(A,2)
        Recfd(:,:,d)=(repmat(A(:,d),[1 length(f)]).*exp(T(:,d)*f)).*repmat(Sf(d,:),[size(A,1),1]);
    end
    Recf=sum(Recfd,3);        
    Q=Recfd.*repmat(conj(Xf-Recf),[1,1,size(Recfd,3)]);
    Hdiag=2*squeeze(sum(repmat((w.*(f.^2)),[size(Q,1),1,size(Q,3)]).*real(Q),2));    
    Hall=zeros(size(A,1),size(A,2),size(A,2));
    for d=1:size(A,2)
        Hall(:,:,d)=2*squeeze(sum(repmat((w.*(f.^2)),[size(Q,1),1,size(Q,3)]).*real(Recfd.*repmat(conj(Recfd(:,:,d)),[1,1,size(A,2)])),2));
    end    
    grad=squeeze(sum(repmat((w.*f),[size(Q,1),1,size(Q,3)]).*(conj(Q)-Q),2));    
    for i=1:size(grad,1)
        grad(i,:)=grad(i,:)/(-diag(Hdiag(i,:))-squeeze(Hall(i,:,:)));
    end

    ind1=find(w==2); % Areas used twice
    ind2=find(w==1); % Areas used once
    cost_old=norm(Xf(:,ind1)-Recf(:,ind1),'fro')^2;
    cost_old=cost_old+0.5*norm(Xf(:,ind2)-Recf(:,ind2),'fro')^2;
    keepgoing=1;
    Told=T;
    while keepgoing
        T=Told-nyT*grad;
        
        for d=1:size(A,2)
            Recfd(:,:,d)=(repmat(A(:,d),[1 length(f)]).*exp(T(:,d)*f)).*repmat(Sf(d,:),[size(A,1),1]);
        end
        Recf=sum(Recfd,3);   
        cost=norm(Xf(:,ind1)-Recf(:,ind1),'fro')^2;
        cost=cost+0.5*norm(Xf(:,ind2)-Recf(:,ind2),'fro')^2;
        if cost<=cost_old
            keepgoing=0;
            nyT=nyT*1.2;
        else
            keepgoing=1;
            nyT=nyT/2;
        end
    end
    T=mod(T,N2);
    ind=find(T>floor(N2/2));
    T(ind)=T(ind)-N2;
    

% -------------------------------------------------------------------------
function [T,A]=estTimeAutCor(Xf,A,Sf,T,f,N)
noc=size(A,2);
if mod(N,2)==0
    sX=2*size(Sf,2)-2;
else
    sX=2*size(Sf,2)-1;
end

Xft=Xf;
t1=randperm(size(Xf,1));
t2=randperm(noc);

for k=t1
  for d=t2
        nocm=setdiff(1:noc,d);
        Xft(k,:)=Xf(k,:)-A(k,nocm)*(Sf(nocm,:).*exp(T(k,nocm)'*f(1,:)));
        C=(conj(Xft(k,:)).*Sf(d,:));
        if mod(N,2)==0
            C=[C conj(C(end-1:-1:2))];
        else
            C=[C conj(C(end:-1:2))];
        end
        C=real(ifft(C));
        [y,ind]=max(abs(C));
        T(k,d)=(ind-sX)-1;
        A(k,d)=C(ind)/((sum(2*(Sf(d,:).*conj(Sf(d,:))))-sum(Sf(d,[1 end]).*conj(Sf(d,[1 end]))))/sX);
        if abs(T(k,d))>(sX/2)
            if T(k,d)>0
                T(k,d)=T(k,d)-sX;
            else 
                 T(k,d)=T(k,d)+sX;
            end
        end                  
  end
end


